#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail import APIError


# TestRail
class TestRailException(APIError):
    pass


class TestRailUserException(TestRailException):
    pass


class TestRailProjectException(TestRailException):
    pass


class TestRailSectionException(TestRailException):
    pass


class TestRailTestSuiteException(TestRailException):
    pass


class TestRailTestCaseException(TestRailException):
    pass


class TestRailTestRunException(TestRailException):
    pass


class TestRailTestPlanException(TestRailException):
    pass


class TestRailTestException(TestRailException):
    pass


class TestRailNewEntityException(TestRailException):
    pass


class TestRailUpdateException(TestRailException):
    pass


class TestRailDeleteException(TestRailException):
    pass


class TestRailSuiteModeException(TestRailException):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import (
    TestRailProjectException,
    TestRailTestRunException,
    TestRailNewEntityException,
    TestRailUpdateException
)

from urllib import error as E
import time


class TestRun:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_test_runs(self, project_id):
        """Get a list of test runs associated with a given project_id.

        :param project_id: project ID of the TestRail project
        :return: response from TestRail API containing the test cases
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_runs/{}".format(project_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test runs. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_runs/{}".format(project_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test runs.")
        finally:
            return result

    def get_test_run(self, run_id):
        """Get a test run by run_id.

        :param run_id: ID of the test run
        :return: response from TestRail API containing the test cases
        """
        if not run_id or run_id is None:
            raise TestRailTestRunException("Invalid run_id")

        if type(run_id) not in [int, float]:
            raise TestRailTestRunException("run_id must be an int or float")

        if run_id <= 0:
            raise TestRailTestRunException("run_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_run/{}".format(run_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test run. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_run/{}".format(run_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test run.")
        finally:
            return result

    def add_test_run(self, project_id, name):
        """Add a test run to a project.

        Supported fields:

            suite_id (int)
                The ID of the test suite for the test run (optional if the project is operating in single suite mode, required otherwise)

            name (string)
                The name of the test run

            description (string)
                The description of the test run

            milestone_id (int)
                The ID of the milestone to link to the test run

            assignedto_id
                int
                The ID of the user the test run should be assigned to

            include_all	(bool)
                True for including all test cases of the test suite and false for a custom case selection (default: true)

            case_ids (array)
                An array of case IDs for the custom case selection

            refs (string)
                A comma-separated list of references/requirements

        :param project_id: ID of the TestRail project
        :param name: name of the test case
        :return: response from TestRail API containing the newly created test run
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id.")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float.")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0.")

        if not name or name is None:
            raise TestRailNewEntityException("Test run name value required.")

        data = dict(name=name, include_all=True)

        result = None
        try:
            result = self.client.send_post("add_run/{}".format(project_id), data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to add test run. Retrying.")
            time.sleep(3)
            try:
                result = self.client.send_post("add_run/{}".format(project_id), data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to add test run.")
        finally:
            return result

    def update_test_run(self, run_id, data):
        """Update a test run in a project.

        Supported fields:

            name (string)
                The name of the test run

            description (string)
                The description of the test run

            milestone_id (int)
                The ID of the milestone to link to the test run

            include_all	(bool)
                True for including all test cases of the test suite and false for a custom case selection (default: true)

            case_ids (array)
                An array of case IDs for the custom case selection

            refs (string)
                A comma-separated list of references/requirements

        :param run_id:
        :param data:
        :return:
        """
        if not run_id or run_id is None:
            raise TestRailTestRunException("Invalid run_id.")

        if type(run_id) not in [int, float]:
            raise TestRailTestRunException("run_id must be an int or float.")

        if run_id <= 0:
            raise TestRailTestRunException("run_id must be > 0.")

        supported_fields = [
            "name",
            "description",
            "milestone_id",
            "include_all",
            "case_ids",
            "refs"
        ]

        if not data or data is None:
            raise TestRailUpdateException("asdfasdf")

        data = self.validate_data(data, supported_fields)

        result = None
        try:
            result = self.client.send_post("update_run/{}".format(run_id), data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to update test run. Retrying.")
            time.sleep(3)
            try:
                result = self.client.send_post("update_run/{}".format(run_id), data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to update test run.")
        finally:
            return result

    def close_test_run(self, run_id):
        """Close out a test run.

        :param run_id:
        :return:
        """
        if not run_id or run_id is None:
            raise TestRailTestRunException("Invalid run_id.")

        if type(run_id) not in [int, float]:
            raise TestRailTestRunException("run_id must be an int or float.")

        if run_id <= 0:
            raise TestRailTestRunException("run_id must be > 0.")

        result = None
        try:
            result = self.client.send_post("close_run/{}".format(run_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to close test run. Retrying.")
            time.sleep(3)
            try:
                result = self.client.send_post("close_run/{}".format(run_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to close test run.")
        finally:
            return result

    def delete_test_run(self, run_id):
        """Delete out a test run.

        :param run_id:
        :return:
        """
        if not run_id or run_id is None:
            raise TestRailTestRunException("Invalid run_id.")

        if type(run_id) not in [int, float]:
            raise TestRailTestRunException("run_id must be an int or float.")

        if run_id <= 0:
            raise TestRailTestRunException("run_id must be > 0.")

        result = None
        try:
            result = self.client.send_post("delete_run/{}".format(run_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to delete test run. Retrying.")
            time.sleep(3)
            try:
                result = self.client.send_post("delete_run/{}".format(run_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to delete test run.")
        finally:
            return result

    @staticmethod
    def validate_data(data_dict, field_list):
        """Field validation static method that I may pull out and use everywhere if it works well.

        :param data_dict:
        :param field_list:
        :return:
        """
        def _valid_key(field):
            return field in field_list

        def _valid_value(value):
            return value is not None and value is not ""

        _valid = dict()
        for k, v in data_dict.items():

            print("[debug] Valid key:\t", _valid_key(k),
                  "\tValid value:\t", _valid_value(v))

            if _valid_key(k) and _valid_value(v):
                _valid[k] = v

        return _valid




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests
import json, base64

"""
	Python class from gurock's repository. Only modifications here are formatting related to be consistent with the rest of my project.
"""


class APIClient:
	"""TestRail API binding for Python 3.x (API v2, available since
	TestRail 3.0)

	Learn more:

		http://docs.gurock.com/testrail-api2/start
		http://docs.gurock.com/testrail-api2/accessing

	Copyright Gurock Software GmbH. See license.md for details.
	"""

	def __init__(self, base_url):
		self.user = ''
		self.password = ''
		if not base_url.endswith('/'):
			base_url += '/'
		self.__url = base_url + 'index.php?/api/v2/'

	def send_get(self, uri, filepath=None):
		"""Issue a GET request (read) against the API.
        Args:
            uri: The API method to call including parameters, e.g. get_case/1.
            filepath: The path and file name for attachment download; used only
                for 'get_attachment/:attachment_id'.
        Returns:
            A dict containing the result of the request.
        """
		return self.__send_request('GET', uri, filepath)

	def send_post(self, uri, data):
		"""Issue a POST request (write) against the API.
        Args:
            uri: The API method to call, including parameters, e.g. add_case/1.
            data: The data to submit as part of the request as a dict; strings
                must be UTF-8 encoded. If adding an attachment, must be the
                path to the file.
        Returns:
            A dict containing the result of the request.
        """
		return self.__send_request('POST', uri, data)

	def __send_request(self, method, uri, data):
		url = self.__url + uri

		auth = str(
			base64.b64encode(
				bytes('%s:%s' % (self.user, self.password), 'utf-8')
			),
			'ascii'
		).strip()
		headers = {'Authorization': 'Basic ' + auth}

		if method == 'POST':
			if uri[:14] == 'add_attachment':  # add_attachment API method
				files = {'attachment': (open(data, 'rb'))}
				response = requests.post(url, headers=headers, files=files)
				files['attachment'].close()
			else:
				headers['Content-Type'] = 'application/json'
				payload = bytes(json.dumps(data), 'utf-8')
				response = requests.post(url, headers=headers, data=payload)
		else:
			headers['Content-Type'] = 'application/json'
			response = requests.get(url, headers=headers)

		if response.status_code > 201:
			try:
				error = response.json()
			except:  # response.content not formatted as JSON
				error = str(response.content)
			raise APIError('TestRail API returned HTTP %s (%s)' % (response.status_code, error))
		else:
			if uri[:15] == 'get_attachment/':  # Expecting file, not JSON
				try:
					open(data, 'wb').write(response.content)
					return (data)
				except:
					return ("Error saving attachment.")
			else:
				try:
					return response.json()
				except:  # Nothing to return
					return {}


class APIError(Exception):
	pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import TestRailProjectException, TestRailTestPlanException, TestRailNewEntityException
from urllib import error as E
import time


class TestPlan:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_test_plans(self, project_id):
        """Get a list of test plans associated with a given project_id.

        :param project_id: project ID of the TestRail project
        :return: response from TestRail API containing the test cases
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_plans/{}".format(project_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test plans. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_plans/{}".format(project_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test plans.")
        finally:
            return result

    def get_test_plan(self, plan_id):
        """Get a test plan by plan_id.

        :param plan_id: ID of the test plan
        :return: response from TestRail API containing the test cases
        """
        if not plan_id or plan_id is None:
            raise TestRailTestPlanException("Invalid plan_id")

        if type(plan_id) not in [int, float]:
            raise TestRailTestPlanException("plan_id must be an int or float")

        if plan_id <= 0:
            raise TestRailTestPlanException("plan_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_plan/{}".format(plan_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test plan. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_plan/{}".format(plan_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test plan.")
        finally:
            return result

    def add_test_plan(self, project_id, name):
        """Add a test plan to a project.

        :param project_id: ID of the TestRail project
        :param name: title of the test plan
        :return: response from TestRail API containing the newly created test plan
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id.")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float.")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0.")

        if not name or name is None:
            raise TestRailNewEntityException("Test plan name value required.")

        data = dict(name=name, include_all=True)

        result = None
        try:
            result = self.client.send_post("add_plan/{}".format(project_id), data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to add test plan. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_post("add_plan/{}".format(project_id), data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to add test plan.")
        finally:
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail import APIClient
from .project import Project
from .section import Section
from .test import Test
from .test_case import TestCase
from .test_plan import TestPlan
from .test_run import TestRun
from .test_suite import TestSuite
from .user import User


class TestRailYak(APIClient):
    """A class to build on top of Gurock's Python interface

    https://github.com/gurock/testrail-api.git
    """
    def __init__(self, url, uname, passwd):

        super().__init__(url)

        self.client             = APIClient(url)
        self.client.user        = uname
        self.client.password    = passwd
        self.project            = Project(self.client)
        self.section            = Section(self.client)
        self.test               = Test(self.client)
        self.test_case          = TestCase(self.client)
        self.test_plan          = TestPlan(self.client)
        self.test_run           = TestRun(self.client)
        self.test_suite         = TestSuite(self.client)
        self.user               = User(self.client)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import TestRailUserException
from urllib import error as E
import time


class User:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_users(self):
        """Get a list of TestRail users.

        :return: response from TestRail API containing the user collection
        """
        result = None
        try:
            result = self.client.send_get("get_users")
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get users. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_users")
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get users.")
        finally:
            return result

    def get_user(self, user_id):
        """Get a TestRail user by user_id.

        :param user_id: user ID of the user we want to grab
        :return: response from TestRail API containing the user
        """
        if not user_id or user_id is None:
            raise TestRailUserException("Invalid user_id")
        result = None
        try:
            result = self.client.send_get("get_user/{}".format(user_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get user. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_user/{}".format(user_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get user.")
        finally:
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import (
    TestRailProjectException,
    TestRailTestSuiteException,
    TestRailNewEntityException
)

from urllib import error as E
import time


class TestSuite:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_test_suites(self, project_id):
        """Get a list of test suites associated with a given project_id.

        :param project_id: project ID of the TestRail project
        :return: response from TestRail API containing the test suites
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_suites/{}".format(project_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test suites. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_suites/{}".format(project_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test suites.")
        finally:
            return result

    def get_test_suite(self, suite_id):
        """Get a test suite by suite_id.

        :param suite_id: ID of the test suite
        :return: response from TestRail API containing the test suites
        """
        if not suite_id or suite_id is None:
            raise TestRailProjectException("Invalid suite_id")

        if type(suite_id) not in [int, float]:
            raise TestRailTestSuiteException("suite_id must be an int or float")

        if suite_id <= 0:
            raise TestRailTestSuiteException("suite_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_suite/{}".format(suite_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test suites. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_suite/{}".format(suite_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test suites.")
        finally:
            return result

    def add_test_suite(self, project_id, name, description):
        """Add a new test suite to a TestRail project.

        :param project_id: ID of the TestRail project
        :param name: name of the new TestRail test suite
        :param description: description of the test suite
        :return: response from TestRail API containing the newly created test suite
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        if not name or name is None:
            raise TestRailNewEntityException("Invalid suite name. Unable to add test suite.")

        if not description or description is None:
            raise TestRailNewEntityException("Invalid description. Unable to add test suite.")

        data = dict(name=name, description=description)

        result = None
        try:
            result = self.client.send_post("add_suite/{}".format(project_id), data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to add test suite. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_post("add_suite/{}".format(project_id), data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to add test suite.")
        finally:
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import setuptools
from distutils.core import setup
from textwrap import dedent


setup(name="testrail_yak",
      version="0.0.4",
      packages=["testrail_yak"],
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
      ],
      license=dedent("""\
      Copyright (c) 2019 bbeale

      Permission is hereby granted, free of charge, to any person obtaining a copy
      of this software and associated documentation files (the "Software"), to deal
      in the Software without restriction, including without limitation the rights
      to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
      copies of the Software, and to permit persons to whom the Software is
      furnished to do so, subject to the following conditions:
      
      The above copyright notice and this permission notice shall be included in all
      copies or substantial portions of the Software.
      
      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
      OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
      SOFTWARE."""),
      description="Python library for interacting with the TestRail API, built on top of gurock's",
      long_description=dedent("""\
      Getting Started:

        To use the TestRail API via TestRailYak, install the package either by downloading the source and running
          $ python setup.py install
        or by using pip
          $ pip install testrail_yak
      """),
      long_description_content_type="text/markdown",
      author="bbeale",
      author_email="beale.ben@gmail.com",
      url="https://github.com/bbeale/TestRailYak",
      setup_requires=["wheel", "requests"],
      requires=["urllib3"]
      )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import (
    TestRailProjectException,
    TestRailSectionException,
    TestRailTestCaseException,
    TestRailNewEntityException
)

from urllib import error as E
import time


class TestCase:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_test_cases(self, project_id):
        """Get a list of test cases associated with a given project_id.

        :param project_id: project ID of the TestRail project
        :return: response from TestRail API containing the test cases
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_cases/{}".format(project_id))    # wtf? [0]
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test cases. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_cases/{}".format(project_id))    # [0]
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test cases.")
        finally:
            return result

    def get_test_case(self, case_id):
        """Get a test case by case_id.

        :param case_id: ID of the test case
        :return: response from TestRail API containing the test cases
        """
        if not case_id or case_id is None:
            raise TestRailTestCaseException("Invalid case_id")

        if type(case_id) not in [int, float]:
            raise TestRailTestCaseException("case_id must be an int or float")

        if case_id <= 0:
            raise TestRailTestCaseException("case_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_case/{}".format(case_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get test case. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_case/{}".format(case_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get test case.")
        finally:
            return result

    def add_test_case(self, section_id, title):
        """Add a test case to a project by section_id.

        :param section_id: ID of the TestRail section
        :param title: title of the test case
        :return: response from TestRail API containing the newly created test case
        """
        if not section_id or section_id is None:
            raise TestRailSectionException("Invalid section_id.")

        if type(section_id) not in [int, float]:
            raise TestRailSectionException("section_id must be an int or float.")

        if section_id <= 0:
            raise TestRailSectionException("section_id must be > 0.")

        if not title or title is None:
            raise TestRailNewEntityException("Test case title required.")

        data = dict(title=title)

        result = None
        try:
            result = self.client.send_post("add_case/{}".format(section_id), data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to add test case. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_post("add_case/{}".format(section_id), data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to add test case.")
        finally:
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .testrail_exception import TestRailProjectException, TestRailNewEntityException
from urllib import error as E
import time


class Project:

    __module__ = "testrail_yak"

    def __init__(self, api):
        self.client = api

    def get_projects(self):
        """Get all projects from the TestRail API."""
        result = None
        try:
            result = self.client.send_get("get_projects")
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get projects. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_projects")
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get projects.")
        finally:
            return result

    def get_project(self, project_id):
        """Get a single project from the TestRail API by passing in its project_id.

        :param project_id: project ID of the TestRail project
        :return: response from TestRail API containing the project
        """
        if not project_id or project_id is None:
            raise TestRailProjectException("Invalid project_id")

        if type(project_id) not in [int, float]:
            raise TestRailProjectException("project_id must be an int or float")

        if project_id <= 0:
            raise TestRailProjectException("project_id must be > 0")

        result = None
        try:
            result = self.client.send_get("get_project/{}".format(project_id))
        except E.HTTPError as httpe:
            print(httpe, "- Failed to get project. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_get("get_project/{}".format(project_id))
            except E.HTTPError as httpe:
                print(httpe, "- Failed to get project.")
        finally:
            return result

    def add_project(self, name, announcement=None, show_announcement=True, suite_mode=1):
        """Add a new project to TestRail.

        :param name: name of the new TestRail project
        :param announcement: brief description of the TestRail project
        :param show_announcement: a truthy value or True show the announcement, a falsey value or False hides it
        :param suite_mode: suite mode of the project (1 for single suite mode, 2 for single suite + baselines, 3 for multiple suites)
        :return: response from TestRail API containing the newly created project
        """
        if not name or name is None:
            raise TestRailNewEntityException("Invalid project name. Unable to create new project.")

        proj_data = dict(
            name                = name,
            announcement        = announcement,
            show_announcement   = show_announcement,
            suite_mode          = suite_mode
        )

        result = None
        try:
            result = self.client.send_post("add_project", proj_data)
        except E.HTTPError as httpe:
            print(httpe, "- Failed to add project. Retrying")
            time.sleep(3)
            try:
                result = self.client.send_post("add_project", proj_data)
            except E.HTTPError as httpe:
                print(httpe, "- Failed to add project.")
        finally:
            return result
